﻿/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

/**
 * @file mxirig.cs : interface of the Moxa IRIGB Card.
 *
 * @version 1.1.0.0 - 2015/07/01
 *
 * @author holsety.chen@moxa.com
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace mxirigcs
{
    [StructLayout(LayoutKind.Sequential)]
    public struct RTCTIME {
        public int nanosec;   /* nano seconds after the second - [0,999999999] */
        public int sec;       /* seconds after the minute - [0,60] */
        public int min;       /* minutes after the hour - [0,59] */
        public int hour;      /* hours since midnight - [0,23] */
        public int mday;      /* day of the month - [1,31] */
        public int mon;       /* months since January - [1,12] */
        public int year;      /* years [00,99] */

        public byte lsp;      /* leap second pending at end of minute */
        public byte ls;       /* leap second type, 0=+, 1=- */
        public byte dsp;      /* daylight saving time change pending at end of minute */
        public byte dst;      /* daylight saving time in effect */
        public byte tzs;      /* Time zone offset sign (0=+) */
        public byte tzh;      /* Time zone offset (0–.5) */
        public byte tz;       /* Time zone offset (0–15) */
        public byte tq;       /* Time quality (Binary, 0–15) */
    };

    public enum _RTC_SYNC_SOURCE_ : int
    {
        TIMESRC_FREERUN = 0,
        TIMESRC_FIBER,
        TIMESRC_PORT1,

        TIMESRC_UNKNOWN
    };

    public enum _IRIG_SIGNAL_STATUS_ : int
    {
        IRIG_STATUS_NORMAL = 0,
        IRIG_STATUS_OFF_LINE,
        IRIG_STATUS_FRAME_ERROR,
        IRIG_STATUS_PARITY_ERROR,

        IRIG_STATUS_UNKNOWN
    };

    public enum _PORT_LIST_ : int
    {
        PORT_FIBER = 0,
        PORT_1,
        PORT_2,
        PORT_3,
        PORT_4,

        PORT_UNKNOWN
    };

    public enum _SIGNAL_TYPE_ : int
    {
        TYPE_TTL = 0,
        TYPE_DIFFERENTIAL,

        TYPE_UNKNOWN
    };

    public enum _OUTPUT_MODE_ : int
    {
        MODE_FROM_FIBER_IN = 0,
        MODE_FROM_PORT1_IN,
        MODE_IRIGB,
        MODE_PPS,

        MODE_UNKNOWN
    };

    public enum _PARITY_CHECK_MODE_ : int
    {
        PARITY_CHECK_EVEN = 0,
        PARITY_CHECK_ODD,
        PARITY_CHECK_NONE
    };

    public enum _IRIGB_BOARD_HWID_ : int
    {
        DA_IRIGB_4DIO_PCI104 = 1,
        DA_IRIGB_S = 7,

        MAX_BOARD_HWID = 8
    };

    /// <summary>
    /// Moxa IRIGB Card Library C Sharp Interface.
    /// </summary>
    public class mxIrig
    {
        const string mxirigDllName = "mxirig.dll";

        /// <summary>
        /// Get Irigb board hardware ID
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="hwId">A pointer to get hardware ID, the value is one of _IRIGB_BOARD_HWID_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetHardwareID(IntPtr hDev, ref int hwId);

        /// <summary>
        /// Open Irigb device
        /// </summary>
        /// <param name="index">the device number (started from 0)</param>
        /// <returns>Pointer to device handle. Return -1 on failure.</returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr mxIrigbOpen(int index);

        /// <summary>
        /// Close Irigb device
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr mxIrigbClose(IntPtr hDev);

        /// <summary>
        /// Get internal RTC time from Irigb device
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="rtctime">A pointer to a RTCTIME structure to receive the current date and time.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetTime(IntPtr hDev, out RTCTIME rtctime);

        /// <summary>
        /// Set internal RTC time to Irigb device
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="rtctime">A pointer to a RTCTIME structure that contains the new date and time.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetTime(IntPtr hDev, ref RTCTIME rtctime);

        /// <summary>
        /// Synchronize local time with internal RTC
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="toFrom">0: Set internal RTC to local time, 1: Set local time to internal RTC</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSyncTime(IntPtr hDev, bool toFrom);

        /// <summary>
        /// Set internal RTC synchronization source
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="rtcSource">internal RTC synchronization source, the value is one of _RTC_SYNC_SOURCE_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetSyncTimeSrc(IntPtr hDev, _RTC_SYNC_SOURCE_ rtcSource);

        /// <summary>
        /// Get RTC synchronization source
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="rtcSource">A point to get internal RTC synchronization source, the value is one of _RTC_SYNC_SOURCE_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetSyncTimeSrc(IntPtr hDev, ref _RTC_SYNC_SOURCE_ rtcSource);

        /// <summary>
        /// Get IRIGB signal status
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="timeSource">IRIGB signal source, the value is one of _PORT_LIST_.</param>
        /// <param name="signalStatus">A point to get IRIGB signal status, the value is one of _IRIG_SIGNAL_STATUS_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetSignalStatus(
            IntPtr hDev, _RTC_SYNC_SOURCE_ timeSource, ref _IRIG_SIGNAL_STATUS_ signalStatus);

        /// <summary>
        /// Set IRIGB input Parity check mode
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="dwSource">the value is one of _RTC_SYNC_SOURCE_, should not be TIMESRC_FREERUN.</param>
        /// <param name="dwMode">the value is one of _PARITY_CHECK_MODE_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        ///
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetInputParityCheckMode(
            IntPtr hDev, _RTC_SYNC_SOURCE_ dwSource, _PARITY_CHECK_MODE_ dwMode);

        /// <summary>
        /// Get IRIGB input Parity check mode
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="dwSource">the value is one of _RTC_SYNC_SOURCE_, should not be TIMESRC_FREERUN.</param>
        /// <param name="pdwMode">A point to get output parity check mode, the value is one of _PARITY_CHECK_MODE_.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        ///
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetInputParityCheckMode(
            IntPtr hDev, _RTC_SYNC_SOURCE_ dwSource, ref _PARITY_CHECK_MODE_ pdwMode);

        /// <summary>
        /// Set IRIGB output Parity check mode
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="dwMode">the value is one of _PARITY_CHECK_MODE_, should not be PARITY_CHECK_NONE.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        ///
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetOutputParityCheckMode(
            IntPtr hDev, _PARITY_CHECK_MODE_ dwMode);

        /// <summary>
        /// Get IRIGB output Parity check mode
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="pdwMode">A point to get output parity check mode.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        ///
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetOutputParityCheckMode(
            IntPtr hDev, ref _PARITY_CHECK_MODE_ pdwMode);

        /// <summary>
        /// Set Pulse Per Second output width
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="pulseWidth">The pulse width per millisecond.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetPpsWidth(IntPtr hDev, int pulseWidth);

        /// <summary>
        /// Get Pulse Per Second output width
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="pulseWidth">A point to get the pulse width per millisecond value.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetPpsWidth(IntPtr hDev, ref int pulseWidth);

        /// <summary>
        /// Set input signal type
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">Signal source, the value is one of _PORT_LIST_.</param>
        /// <param name="type">Signal type, the value is one of _SIGNAL_TYPE_.</param>
        /// <param name="invert">Signal inverse mode, if nonzero, invert the input signal.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetInputSignalType(
            IntPtr hDev, _PORT_LIST_ port, _SIGNAL_TYPE_ type, bool invert);

        /// <summary>
        /// Get input signal type
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">Signal source, the value is one of _PORT_LIST_.</param>
        /// <param name="type">A point to get IRIGB signal type, the value is one of _SIGNAL_TYPE_.</param>
        /// <param name="invert">A point to get signal mode, if nonzero, the signal is inverse.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetInputSignalType(
            IntPtr hDev, _PORT_LIST_ port, ref _SIGNAL_TYPE_ type, ref bool invert);


        /// <summary>
        /// Set output signal type
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">Signal source, the value is one of _PORT_LIST_.</param>
        /// <param name="type">Signal type, the value is one of _SIGNAL_TYPE_.</param>
        /// <param name="mode">Signal output mode, the value is one of _OUTPUT_MODE_.</param>
        /// <param name="invert">Signal inverse mode, if nonzero, invert the output signal.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetOutputSignalType(
            IntPtr hDev, _PORT_LIST_ port, _SIGNAL_TYPE_ type, _OUTPUT_MODE_ mode, bool invert);

        /// <summary>
        /// Get output signal type
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">Signal source, the value is one of _PORT_LIST_.</param>
        /// <param name="type">A point to get signal type, the value is one of _SIGNAL_TYPE_.</param>
        /// <param name="mode">A point to get signal output mode, the value is one of _OUTPUT_MODE_.</param>
        /// <param name="invert">A point to get signal mode, if nonzero, the signal is inverse.</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetOutputSignalType(
            IntPtr hDev, _PORT_LIST_ port, ref _SIGNAL_TYPE_ type, ref _OUTPUT_MODE_ mode, ref bool invert);

        /// <summary>
        /// Set Digital Output Signal
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">the port number (started from 0)</param>
        /// <param name="value">the port data, 1:HIGH, 0:LOW</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbSetDigitalOutputSignal(IntPtr hDev, int port, int value);

        /// <summary>
        /// Get Digital Output Signal
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">the port number (started from 0)</param>
        /// <param name="value">A point to get port data, 1:HIGH, 0:LOW</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetDigitalOutputSignal(IntPtr hDev, int port, ref int value);

        /// <summary>
        /// Get Digital Input Signal
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="port">the port number (started from 0)</param>
        /// <param name="value">A point to get port data, 1:HIGH, 0:LOW</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetDigitalInputSignal(IntPtr hDev, int port, ref int value);

        /// <summary>
        /// Get FPGA firmware build date
        /// </summary>
        /// <param name="hDev">A valid handle value return from "mxIrigbOpen" function.</param>
        /// <param name="value">A point to get FPGA firmware build date, the value is BCD style yyyyMMdd, eg.:0x20140101</param>
        /// <returns>
        /// If the operation completes successfully, the return value is nonzero.
        /// If the operation fails or is pending, the return value is zero.
        /// To get extended error information, call GetLastError.
        /// </returns>
        /// 
        [DllImport(mxirigDllName, SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool mxIrigbGetFpgaBuildDate(IntPtr hDev, ref uint value);
    }
}
